/* Copyright 1998-2011 Research Foundation State University of New York */

/* This file is part of QuB.                                            */

/* QuB is free software; you can redistribute it and/or modify          */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation, either version 3 of the License, or    */
/* (at your option) any later version.                                  */

/* QuB is distributed in the hope that it will be useful,               */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/* GNU General Public License for more details.                         */

/* You should have received a copy of the GNU General Public License,   */
/* named LICENSE.txt, in the QuB program directory.  If not, see        */
/* <http://www.gnu.org/licenses/>.                                      */

#ifndef QUB_KALMAN_H
#define QUB_KALMAN_H

#include "ublas_plus.h"
#include "maxill.h"

#ifdef __cplusplus
extern "C" {
#endif


typedef struct {
	void *storage;
	int Nprocess, Nmeasure;
	double *X;     // posterior process vector
	double *Z;     // posterior measurement vector (filter output for last sample)
	double **P;    // posterior process covariance
	double **A;    // process matrix
	double **Q;    // process covariance
	double **H;    // process->measurement matrix
	double **R;    // measurement covariance
} qub_kalman_filter;

MAXILL_API qub_kalman_filter * qub_kalman_filter_create(int Nprocess, int Nmeasure);
MAXILL_API void qub_kalman_filter_free(qub_kalman_filter *filter);
MAXILL_API void qub_kalman_filter_next(qub_kalman_filter *filter, double *z);
MAXILL_API void qub_kalman_filter_reset(qub_kalman_filter *filter);


#ifdef __cplusplus
}
#endif

#endif
